@@ -1,10 +1,10 @@
-#' @title Function factory for value formatting
+#' @title Function factory for formatting values
 #'
 #' @description \code{format_valuef} generates functions that format
 #'   values so that they print out with the desired number of digits.
 #'
 #' @param digits number of digits to use
-#' @return returns a function that takes an atomic vector as argument
+#' @return Returns a function that takes an atomic vector as its argument
 #'   and returns it formatted to character with \code{digits} decimals.
 
 format_valuef <- function(digits) {
@@ -17,8 +17,10 @@ format_valuef <- function(digits) {
 #' @inheritParams plot.gsmar
 #' @param digits number of digits to be printed (max 20)
 #' @param summary_print if set to \code{TRUE} then the print will include approximate
-#'  standard errors, log-likelihood and information criteria values. Supported only for
-#'  models with data.
+#'  standard errors for the estimates, log-likelihood, information criteria values,
+#'  modulus of the roots of the characteristic AR polynomias for each regime, and
+#'  several unconditional moments.
+#'  Supported for models that include data only.
 #' @export
 
 print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
@@ -44,8 +46,8 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
   model <- gsmar$model$model
   restricted <- gsmar$model$restricted
   constraints <- gsmar$model$constraints
-  all_mu <- round(get_regime_means(gsmar), digits)
-  #all_vars <- round(get_regime_vars(gsmar), digits) uncomment to print regime variances
+  all_mu <- get_regime_means(gsmar)
+  if(summary_print) all_vars <- get_regime_vars(gsmar)  # Unconditional regime variances
 
   if(gsmar$model$parametrization == "mean") {
     params <- change_parametrization(p=p, M=M, params=params, model=model, restricted=restricted,
@@ -88,10 +90,12 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
 
   if(summary_print) {
     IC <- gsmar$IC
-    cat(paste("\nlog-likelihood:", format_value(gsmar$loglik)), "\n")
-    cat(paste("AIC: ", format_value(IC$AIC)), "\n")
-    cat(paste("HQIC:", format_value(IC$HQIC)), "\n")
-    cat(paste("BIC: ", format_value(IC$BIC)), "\n")
+    form_val2 <- function(txt, val) paste(txt, format_value(val))
+    cat("\n", paste(form_val2("log-likelihood:", gsmar$loglik),
+                    form_val2("AIC:", IC$AIC),
+                    form_val2("HQIC:", IC$HQIC),
+                    form_val2("BIC:", IC$BIC),
+                    sep=", "), "\n")
   }
 
   for(m in seq_len(sum(M))) {
@@ -117,19 +121,17 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
     print_err(alphas_err[m])
     cat("\nReg mean:", format_value(all_mu[m]))
     print_err(mu_err[m])
-    #cat("\nReg var:", format_value(all_vars[m])) # Uncomment to print regime variances
-    cat("\n")
 
     if(regime_type == "StMAR") {
-      cat("Var param:", format_value(pars[nrow(pars), m]))
+      cat("\nVar param:", format_value(pars[nrow(pars), m]))
       print_err(pars_err[nrow(pars_err), m])
       cat("\n")
 
       cat("Df param:", format_value(dfs[m - M1]))
       print_err(dfs_err[m - M1])
-      cat("\n")
     }
-    cat("\n")
+    if(summary_print) cat("\nReg var: ", format_value(all_vars[m])) # Unconditional regime variances
+    cat("\n\n")
 
     cat(paste0("Y = [", format_value(all_phi0[m]), "]"))
     add_string(const_spaces=4, all_phi0[m], phi0_err[m])
@@ -164,16 +166,17 @@ print.gsmar <- function(x, ..., digits=2, summary_print=FALSE) {
 
 #' @title Print method from objects of class 'gsmarsum'
 #'
-#' @description \code{print.gsmarsum} is a print method for object \code{'gsmarsum'} created
-#'  with \code{summary.gsmar}. Approximate standard errors are printed in brackets.
+#' @description \code{print.gsmarsum} is a print method for objects of class 'gsmarsum'
+#'  created with the summary method \code{summary.gsmar}. Approximate standard errors
+#'  are printed in brackets.
 #'
 #' @param x object of class 'gsmarsum' generated by \code{summary.gsmar}.
-#' @param ... currently not used.
-#' @param digits number of digits to be printed
+#' @param ... currently not in use.
+#' @param digits the number of digits to be printed
 #' @examples
 #' \donttest{
 #' # GMAR model
-#' fit12 <- fitGSMAR(VIX, 1, 2, nCalls=4)
+#' fit12 <- fitGSMAR(logVIX, 1, 2, ncalls=4)
 #' gsmarsum12 <- summary(fit12)
 #' gsmarsum12
 #' print(gsmarsum12, digits=4)
@@ -187,14 +190,13 @@ print.gsmarsum <- function(x, ..., digits) {
 
 
 
-#' @title print method for class 'gsmarpred' objects
+#' @title Print method for class 'gsmarpred' objects
 #'
-#' @description \code{print.gsmarpred} is print method for object created
-#'  by \code{predict.gsmar}.
+#' @description \code{print.gsmarpred} is a print method for call 'gsmarpred'
+#'  objects created with \code{predict.gsmar}.
 #'
+#' @inheritParams print.gsmarsum
 #' @param x object of class \code{'gsmarpred'} generated by \code{predict.gsmar}.
-#' @param digits number of decimals to print
-#' @param ... currently not used.
 #' @examples
 #'  \donttest{
 #'  # GMAR-model
@@ -212,18 +214,18 @@ print.gsmarpred <- function(x, ..., digits=2) {
   format_value <- format_valuef(digits)
 
   if(gsmarpred$pred_type == "cond_mean") {
-    cat("One-step-ahead prediction by exact conditional mean, no prediction intervals\n")
+    cat("One-step-ahead prediction by exact conditional mean, no prediction intervals.\n")
     cat("Forecast:", paste0(format_value(gsmarpred$pred), collapse=", "), "\n")
 
   } else if(gsmarpred$pi_type == "none") {
-    cat(paste0("Prediction by ", gsmarpred$pred_type, ", no prediction intervals"), "\n")
-    cat(paste0("Forecast ", gsmarpred$n_ahead, " steps ahead, based on ", gsmarpred$n_simu, " simulations\n"))
+    cat(paste0("Prediction by ", gsmarpred$pred_type, ", no prediction intervals."), "\n")
+    cat(paste0("Forecast ", gsmarpred$n_ahead, " steps ahead, based on ", gsmarpred$nsimu, " simulations.\n"))
     print(data.frame(pred=gsmarpred$pred))
 
   } else {
     cat(paste0("Prediction by ", gsmarpred$pred_type, ", ", gsmarpred$pi_type,
-               " prediction intervals with levels ", paste(gsmarpred$pi, collapse=", "), ""), "\n")
-    cat(paste0("Forecast ", gsmarpred$n_ahead, " steps ahead, based on ", gsmarpred$nsimu, " simulations\n"))
+               " prediction intervals with levels ", paste(gsmarpred$pi, collapse=", "), "."), "\n")
+    cat(paste0("Forecast ", gsmarpred$n_ahead, " steps ahead, based on ", gsmarpred$nsimu, " simulations.\n"))
 
     cat("\n")
     q <- gsmarpred$q
@@ -245,11 +247,11 @@ print.gsmarpred <- function(x, ..., digits=2) {
 }
 
 
-#' @describeIn quantileResidualTests print method for class 'qrtest'
+#' @describeIn quantileResidualTests Print method for class 'qrtest' objects
 #' @param x object of class \code{'qrtest'} created with the function \code{quantileResidualTests}.
 #' @param ... graphical parameters passed to \code{segments} in \code{plot.qrtest}.
 #'  Currectly not used in \code{print.qrtest}
-#' @param digits number of decimals to print
+#' @param digits the number of digits to be print
 #' @export
 print.qrtest <- function(x, ..., digits=3) {
   qrtest <- x
@@ -257,21 +259,20 @@ print.qrtest <- function(x, ..., digits=3) {
   format_lag <- format_valuef(0)
   cat(paste("Normality test p-value:", format_value(qrtest$norm_res$pvalue)), "\n\n")
 
+  # Function to print autocorrelation and cond. h.sked test results
+  print_res <- function(which_res=c("ac", "ch")) {
+    wres <- ifelse(which_res == "ac", 2, 3) # Index in the list 'qrtest'
+    lags <- qrtest[[wres]]$lags
+    for(i1 in seq_along(lags)) { # Go through tested lags
+      sep <- ifelse(lags[i1] < 10, " | ", "| ")
+      cat(" ", format_lag(lags[i1]), sep, format_value(qrtest[[wres]]$pvalue[i1]), "\n")
+    }
+  }
+
   cat("Autocorrelation tests:\nlags | p-value\n")
-  for(i1 in seq_along(qrtest$ac_res$lags)) {
-    if(qrtest$ac_res$lags[i1] < 10) {
-      cat(" ", format_lag(qrtest$ac_res$lags[i1]), " | ", format_value(qrtest$ac_res$pvalue[i1]), "\n")
-    } else {
-      cat(" ", format_lag(qrtest$ac_res$lags[i1]), "| ", format_value(qrtest$ac_res$pvalue[i1]), "\n")
-    }
-  }
+  print_res("ac")
+
   cat("\nConditional hetetoskedasticity tests:\nlags | p-value\n")
-  for(i1 in seq_along(qrtest$ch_res$lags)) {
-    if(qrtest$ch_res$lags[i1] < 10) {
-      cat(" ", format_lag(qrtest$ch_res$lags[i1]), " | ", format_value(qrtest$ch_res$pvalue[i1]), "\n")
-    } else {
-      cat(" ", format_lag(qrtest$ch_res$lags[i1]), "| ", format_value(qrtest$ch_res$pvalue[i1]), "\n")
-    }
-  }
+  print_res("ch")
 }
 
